@@ -8,11 +8,12 @@ package Parser::MGC;
 use strict;
 use warnings;
 
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
 use Carp;
 
 use File::Slurp::Tiny qw( read_file );
+use Scalar::Util qw( blessed );
 
 =head1 NAME
 
@@ -53,10 +54,27 @@ the token parsing behaviour.
 It provides a number of token-parsing methods, which each extract a
 grammatical token from the string. It also provides wrapping methods that can
 be used to build up a possibly-recursive grammar structure, by applying a
-structure around other parts of parsing code. Each method, both token and
-structural, atomically either consumes a prefix of the string and returns its
-result, or fails and consumes nothing. This makes it simple to implement
-grammars that require backtracking.
+structure around other parts of parsing code.
+
+=head2 Backtracking
+
+Each method, both token and structural, atomically either consumes a prefix of
+the string and returns its result, or fails and consumes nothing. This makes
+it simple to implement grammars that require backtracking.
+
+Several structure-forming methods have some form of "optional" behaviour; they
+can optionally consume some amount of input or take some particular choice,
+but if the code invoked inside that subsequently fails, the structure can
+backtrack and take some different behaviour. This is usually what is required
+when testing whether the structure of the input string matches some part of
+the grammar that is optional, or has multiple choices.
+
+However, once the choice of grammar has been made, it is often useful to be
+able to fix on that one choice, thus making subsequent failures propagate up
+rather than taking that alternative behaviour. Control of this backtracking
+is given by the C<commit> method; and careful use of this method is one of the
+key advantages that C<Parser::MGC> has over more simple parsing using single
+regexps alone.
 
 =cut
 
@@ -64,7 +82,9 @@ grammars that require backtracking.
 
 =cut
 
-=head2 $parser = Parser::MGC->new( %args )
+=head2 new
+
+   $parser = Parser::MGC->new( %args )
 
 Returns a new instance of a C<Parser::MGC> object. This must be called on a
 subclass that provides method of the name provided as C<toplevel>, by default
@@ -178,7 +198,9 @@ sub new
 
 =cut
 
-=head2 $result = $parser->from_string( $str )
+=head2 from_string
+
+   $result = $parser->from_string( $str )
 
 Parse the given literal string and return the result from the toplevel method.
 
@@ -202,7 +224,9 @@ sub from_string
    return $result;
 }
 
-=head2 $result = $parser->from_file( $file, %opts )
+=head2 from_file
+
+   $result = $parser->from_file( $file, %opts )
 
 Parse the given file, which may be a pathname in a string, or an opened IO
 handle, and return the result from the toplevel method.
@@ -234,7 +258,11 @@ sub from_file
       ( read_file $file, binmode => $opts{binmode} ) );
 }
 
-=head2 $result = $parser->from_reader( \&reader )
+=head2 from_reader
+
+   $result = $parser->from_reader( \&reader )
+
+I<Since version 0.05.>
 
 Parse the input which is read by the C<reader> function. This function will be
 called in scalar context to generate portions of string to parse, being passed
@@ -272,7 +300,11 @@ sub from_reader
    return $result;
 }
 
-=head2 $pos = $parser->pos
+=head2 pos
+
+   $pos = $parser->pos
+
+I<Since version 0.09.>
 
 Returns the current parse position, as a character offset from the beginning
 of the file or string.
@@ -285,7 +317,9 @@ sub pos
    return pos $self->{str};
 }
 
-=head2 ( $lineno, $col, $text ) = $parser->where
+=head2 where
+
+   ( $lineno, $col, $text ) = $parser->where
 
 Returns the current parse position, as a line and column number, and
 the entire current line of text. The first line is numbered 1, and the first
@@ -317,14 +351,24 @@ sub where
    return ( $lineno, $col, $line );
 }
 
-=head2 $parser->fail( $message )
+=head2 fail
 
-=head2 $parser->fail_from( $pos, $message )
+=head2 fail_from
+
+   $parser->fail( $message )
+
+   $parser->fail_from( $pos, $message )
+
+I<C<fail_from> since version 0.09.>
 
 Aborts the current parse attempt with the given message string. The failure
 message will include the line and column position, and the line of input that
-failed at the current parse position, or a position earlier obtained using the
-C<pos> method.
+failed at the current parse position (C<fail>), or a position earlier obtained
+using the C<pos> method (C<fail_from>).
+
+This failure will propagate up to the inner-most structure parsing method that
+has not been committed; or will cause the entire parser to fail if there are
+no further options to take.
 
 =cut
 
@@ -342,7 +386,11 @@ sub fail_from
    die Parser::MGC::Failure->new( $message, $self, $pos );
 }
 
-=head2 $eos = $parser->at_eos
+sub _isa_failure { blessed $_[0] and $_[0]->isa( "Parser::MGC::Failure" ) }
+
+=head2 at_eos
+
+   $eos = $parser->at_eos
 
 Returns true if the input string is at the end of the string.
 
@@ -373,7 +421,11 @@ sub at_eos
    return $at_eos;
 }
 
-=head2 $level = $parser->scope_level
+=head2 scope_level
+
+   $level = $parser->scope_level
+
+I<Since version 0.05.>
 
 Returns the number of nested C<scope_of> calls that have been made.
 
@@ -393,12 +445,20 @@ which will be passed the actual C<$parser> object as its first argument.
 
 =cut
 
-=head2 $ret = $parser->maybe( $code )
+=head2 maybe
+
+   $ret = $parser->maybe( $code )
 
 Attempts to execute the given C<$code> reference in scalar context, and
-returns what it returned. If the code fails to parse by calling the C<fail>
-method then none of the input string will be consumed; the current parsing
-position will be restored. C<undef> will be returned in this case.
+returns what it returned, accepting that it might fail.
+
+If the code fails (either by calling C<fail> itself, or by propagating a
+failure from another method it invoked) before it has invoked C<commit>, then
+none of the input string will be consumed; the current parsing position will
+be restored. C<undef> will be returned in this case.
+
+If it calls C<commit> then any subsequent failure will be propagated to the
+caller, rather than returning C<undef>.
 
 This may be considered to be similar to the C<?> regexp qualifier.
 
@@ -433,11 +493,13 @@ sub maybe
 
    pos($self->{str}) = $pos;
 
-   die $e if $committed or not eval { $e->isa( "Parser::MGC::Failure" ) };
+   die $e if $committed or not _isa_failure( $e );
    return undef;
 }
 
-=head2 $ret = $parser->scope_of( $start, $code, $stop )
+=head2 scope_of
+
+   $ret = $parser->scope_of( $start, $code, $stop )
 
 Expects to find the C<$start> pattern, then attempts to execute the given
 C<$code> reference, then expects to find the C<$stop> pattern. Returns
@@ -465,9 +527,12 @@ based on the start pattern.
     my $delim = $self->expect( qr/[\(\[\<\{]/ );
     $delim =~ tr/([<{/)]>}/;
 
-    $self->enter_scope( undef, sub { $self->parse_body }, $delim );
+    $self->scope_of( undef, sub { $self->parse_body }, $delim );
  }
 
+This method does not have any optional parts to it; any failures are
+immediately propagated to the caller.
+
 =cut
 
 sub scope_of
@@ -489,11 +554,14 @@ sub scope_of
    return $ret;
 }
 
-=head2 $ret = $parser->list_of( $sep, $code )
+=head2 list_of
+
+   $ret = $parser->list_of( $sep, $code )
 
 Expects to find a list of instances of something parsed by C<$code>,
 separated by the C<$sep> pattern. Returns an ARRAY ref containing a list of
-the return values from the C<$code>.
+the return values from the C<$code>. A single trailing delimiter is allowed,
+and does not affect the return value.
 
 This method does not consider it an error if the returned list is empty; that
 is, that the scope ended before any item instances were parsed from it.
@@ -505,6 +573,15 @@ is, that the scope ended before any item instances were parsed from it.
     $self->list_of( ",", sub { $self->token_int } );
  }
 
+If the code fails (either by invoking C<fail> itself, or by propagating a
+failure from another method it invoked) before it has invoked C<commit> on a
+particular item, then the item is aborted and the parsing position will be
+restored to the beginning of that failed item. The list of results from
+previous successful attempts will be returned.
+
+If it calls C<commit> within an item then any subsequent failure for that item
+will cause the entire C<list_of> to fail, propagating that to the caller.
+
 =cut
 
 sub list_of
@@ -527,7 +604,8 @@ sub list_of
       my $e = $@;
 
       pos($self->{str}) = $pos;
-      die $e if $committed or not eval { $e->isa( "Parser::MGC::Failure" ) };
+      die $e if $committed or not _isa_failure( $e );
+
       last;
    }
    continue {
@@ -540,7 +618,9 @@ sub list_of
    return \@ret;
 }
 
-=head2 $ret = $parser->sequence_of( $code )
+=head2 sequence_of
+
+   $ret = $parser->sequence_of( $code )
 
 A shortcut for calling C<list_of> with an empty string as separator; expects
 to find at least one instance of something parsed by C<$code>, separated only
@@ -555,6 +635,9 @@ This may be considered to be similar to the C<+> or C<*> regexp qualifiers.
     $self->sequence_of( sub { $self->parse_statement } );
  }
 
+The interaction of failures in the code and the C<commit> method is identical
+to that of C<list_of>.
+
 =cut
 
 sub sequence_of
@@ -565,11 +648,15 @@ sub sequence_of
    $self->list_of( undef, $code );
 }
 
-=head2 $ret = $parser->any_of( @codes )
+=head2 any_of
+
+   $ret = $parser->any_of( @codes )
+
+I<Since version 0.06.>
 
 Expects that one of the given code references can parse something from the
 input, returning what it returned. Each code reference may indicate a failure
-to parse by calling the C<fail> method.
+to parse by calling the C<fail> method or otherwise propagating a failure.
 
 This may be considered to be similar to the C<|> regexp operator for forming
 alternations of possible parse trees.
@@ -585,10 +672,14 @@ alternations of possible parse trees.
     );
  }
 
-Note: This method used to be called C<one_of>, but was renamed for clarity.
-Currently this method is provided also as an alias by the old name. Code
-using the old name should be rewritten to C<any_of> instead, as this 
-backward-compatibility alias may be removed in a later version.
+If the code for a given choice fails (either by invoking C<fail> itself, or by
+propagating a failure from another method it invoked) before it has invoked
+C<commit> itself, then the parsing position restored and the next choice will
+be attempted.
+
+If it calls C<commit> then any subsequent failure for that choice will cause
+the entire C<any_of> to fail, propagating that to the caller and no further
+choices will be attmepted.
 
 =cut
 
@@ -608,24 +699,30 @@ sub any_of
 
       pos( $self->{str} ) = $pos;
 
-      die $e if $committed or not eval { $e->isa( "Parser::MGC::Failure" ) };
+      die $e if $committed or not _isa_failure( $e );
    }
 
    $self->fail( "Found nothing parseable" );
 }
 
-*one_of = \&any_of;
+sub one_of {
+   carp "Parser::MGC->one_of is deprecated; use ->any_of instead";
+   goto &any_of;
+}
 
-=head2 $parser->commit
+=head2 commit
+
+   $parser->commit
 
 Calling this method will cancel the backtracking behaviour of the innermost
 C<maybe>, C<list_of>, C<sequence_of>, or C<any_of> structure forming method.
 That is, if later code then calls C<fail>, the exception will be propagated
-out of C<maybe>, and no further code blocks will be attempted by C<any_of>.
+out of C<maybe>, no further list items will be attempted by C<list_of> or
+C<sequence_of>, and no further code blocks will be attempted by C<any_of>.
 
-Typically this will be called once the grammatical structure of an
-alternation has been determined, ensuring that any further failures are raised
-as real exceptions, rather than by attempting other alternatives.
+Typically this will be called once the grammatical structure alter has been
+determined, ensuring that any further failures are raised as real exceptions,
+rather than by attempting other alternatives.
 
  sub parse_statement
  {
@@ -690,11 +787,13 @@ sub skip_ws
    }
 }
 
-=head2 $str = $parser->expect( $literal )
+=head2 expect
 
-=head2 $str = $parser->expect( qr/pattern/ )
+   $str = $parser->expect( $literal )
 
-=head2 @groups = $parser->expect( qr/pattern/ )
+   $str = $parser->expect( qr/pattern/ )
+
+   @groups = $parser->expect( qr/pattern/ )
 
 Expects to find a literal string or regexp pattern match, and consumes it.
 In scalar context, this method returns the string that was captured. In list
@@ -707,9 +806,13 @@ example C<qr/\d*/>), the pattern will always match, even if it has to match an
 empty string. This method will not consider a failure if the regexp matches
 with zero-width.
 
-=head2 $str = $parser->maybe_expect( ... )
+=head2 maybe_expect
 
-=head2 @groups = $parser->maybe_expect( ... )
+   $str = $parser->maybe_expect( ... )
+
+   @groups = $parser->maybe_expect( ... )
+
+I<Since version 0.10.>
 
 A convenient shortcut equivalent to calling C<expect> within C<maybe>, but
 implemented more efficiently, avoiding the exception-handling set up by
@@ -750,9 +853,13 @@ sub expect
    }
 }
 
-=head2 $str = $parser->substring_before( $literal )
+=head2 substring_before
 
-=head2 $str = $parser->substring_before( qr/pattern/ )
+   $str = $parser->substring_before( $literal )
+
+   $str = $parser->substring_before( qr/pattern/ )
+
+I<Since version 0.06.>
 
 Expects to possibly find a literal string or regexp pattern match. If it finds
 such, consume all the input text before but excluding this match, and return
@@ -806,7 +913,11 @@ sub substring_before
    return substr( $self->{str}, $start, $end - $start );
 }
 
-=head2 $val = $parser->generic_token( $name, $re, $convert )
+=head2 generic_token
+
+   $val = $parser->generic_token( $name, $re, $convert )
+
+I<Since version 0.08.>
 
 Expects to find a token matching the precompiled regexp C<$re>. If provided,
 the C<$convert> CODE reference can be used to convert the string into a more
@@ -859,7 +970,9 @@ sub _token_generic
    $self->generic_token( $name, $re, $convert );
 }
 
-=head2 $int = $parser->token_int
+=head2 token_int
+
+   $int = $parser->token_int
 
 Expects to find an integer in decimal, octal or hexadecimal notation, and
 consumes it. Negative integers, preceeded by C<->, are also recognised.
@@ -885,7 +998,11 @@ sub token_int
    );
 }
 
-=head2 $float = $parser->token_float
+=head2 token_float
+
+   $float = $parser->token_float
+
+I<Since version 0.04.>
 
 Expects to find a number expressed in floating-point notation; a sequence of
 digits possibly prefixed by C<->, possibly containing a decimal point,
@@ -905,7 +1022,11 @@ sub token_float
    );
 }
 
-=head2 $number = $parser->token_number
+=head2 token_number
+
+   $number = $parser->token_number
+
+I<Since version 0.09.>
 
 Expects to find a number expressed in either of the above forms.
 
@@ -917,7 +1038,9 @@ sub token_number
    $self->any_of( \&token_float, \&token_int );
 }
 
-=head2 $str = $parser->token_string
+=head2 token_string
+
+   $str = $parser->token_string
 
 Expects to find a quoted string, and consumes it. The string should be quoted
 using C<"> or C<'> quote marks.
@@ -988,7 +1111,9 @@ sub token_string
    return $string;
 }
 
-=head2 $ident = $parser->token_ident
+=head2 token_ident
+
+   $ident = $parser->token_ident
 
 Expects to find an identifier, and consumes it.
 
@@ -1004,7 +1129,9 @@ sub token_ident
    );
 }
 
-=head2 $keyword = $parser->token_kw( @keywords )
+=head2 token_kw
+
+   $keyword = $parser->token_kw( @keywords )
 
 Expects to find a keyword, and consumes it. A keyword is defined as an
 identifier which is exactly one of the literal values passed in.
